using System.Collections.Generic;
using System.Threading.Tasks;
using WingInterfaceLibrary.DTO.Education;
using WingInterfaceLibrary.Request.Education;
using WingInterfaceLibrary.Request.Courses;
using WingInterfaceLibrary.Result;
using WingInterfaceLibrary.DTO.Courses;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Result.Education;
using WingInterfaceLibrary.Result.Courses;

namespace WingInterfaceLibrary.Interface
{
    /// <summary>
    /// 教培服务接口
    /// </summary>
    public interface IEducationService
    {
        /// <summary>
        /// 获取标签列表 
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns>标签信息</returns>
        /// <errorCodes></errorCodes>
        Task<IList<CourseLabelDTO>> GetCourseLabelsAsync(QueryCourseLabelListRequest request);

        /// <summary>
        /// 申请课程
        /// </summary>
        /// <param name="request">申请课程请求</param>
        /// <returns>成功或者失败</returns>
        /// <errorCodes>TeacherCodeEmpty,TeacherNoFound,UserNotFind </errorCodes>
        Task<bool> ApplyCourseAsync(ApplyCourseRequest request);

        /// <summary>
        /// 查询自己申请课程列表分页
        /// </summary>
        /// <param name="request">查询自己申请课程请求</param>
        /// <returns>课程列表信息</returns>
        ///  <errorCodes>UserNotFind</errorCodes>
        Task<PageResult<CoursePageDTO>> FindCoursePagesAsync(FindCoursePagesRequest request);

        /// <summary>
        /// 删除课程记录
        /// </summary>
        /// <param name="request">删除课程请求</param>
        /// <returns>删除成功或者失败</returns>
        /// <errorCodes>UserNotFind,CourseCodeEmpty,CourseNoFound,NotDeleteForCourseStatus</errorCodes>
        Task<bool> DeleteCourseByCodeAsync(DeleteCourseByCodeRequest request);

        /// <summary>
        /// 更新课程记录
        /// </summary>
        /// <param name="request">更新课程请求</param>
        /// <returns>更新成功或者失败</returns>
        ///  <errorCodes>UserNotFind,CourseCodeEmpty,TeacherNoFound,CourseNoFound,NotUpdateForCourseStatus </errorCodes>
        Task<bool> UpdateCourseAsync(UpdateCourseRequest request);

        /// <summary>
        /// 根据课程编码获取课程详细信息
        /// </summary>
        /// <param name="request">获取详细信息课程请求</param>
        /// <returns>课程详细信息</returns>
        ///  <errorCodes>UserNotFind,CourseCodeEmpty</errorCodes>
        Task<CourseInfoDetailDTO> FindCourseByCodeAsync(FindCourseByCodeRequest request);

        /// <summary>
        /// 查询自己当学生的课程列表分页
        /// </summary>
        /// <param name="request">学生查询自己课程请求</param>
        /// <returns>课程列表</returns>
        /// <errorCodes>UserNotFind</errorCodes>
        Task<PageResult<StudentInCoursePageDTO>> FindStudentInCoursePagesAsync(FindStudentInCoursePagesRequest request);

        /// <summary>
        /// 客户端根据创建人获取创建的用户组
        /// </summary> 
        /// <param name="request">创建人code</param>
        /// <returns></returns>
        /// <errorCodes>UserNotFind</errorCodes>
        Task<IList<UserGroupDTO>> GetUserGroupListsByCreatorCodeAsync(TokenRequest request);

        /// <summary>
        /// 新增用户组
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        /// <errorCodes>UserNotFind,UserGroupNameIsEmpty,UserGroupMaxPeopleNotValid</errorCodes>
        Task<bool> AddUserGroupAsync(UserGroupRequest request);

        /// <summary>
        /// 查询用户分页
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        /// <errorCodes>UserGroupCodeIsEmpty</errorCodes>
        Task<PageResult<StudentInfoDTO>> FindUserGroupStudentsPagesAsync(QueryStudentByGroupCodePageRequest request);

        /// <summary>
        /// 用户组移除用户
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        /// <errorCodes>UserGroupCodeIsEmpty,UserGroupStudentsIsEmpty</errorCodes>
        Task<bool> RemoveStudentsByGroupCodeAsync(UserGroupFilterRequest request);

        /// <summary>
        /// 创建视频
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> CreateVideoAsync(SaveVideoRequest request);

        /// <summary>
        /// 查询所有视频列表分页
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<PageResult<VideoInfoDTO>> FindVideosPagesAsync(FindVideoPagesRequest request);

        /// <summary>
        /// 删除视频，同步删除课程管理视频code
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> DeleteVideoByCodeAsync(DeleteVideoRequest request);

        /// <summary>
        /// 购买课程
        /// </summary>
        /// <param name="request">购买课程请求实体</param>
        /// <returns></returns>
        /// <errorCodes>CourseNoFound,CourseNotReviewed,CourseNoPayRequired,CourseQuantityError,CreateCourseOrderFailed</errorCodes>
        Task<BuyCourseResult> BuyCourseAsync(BuyCourseRequest request);

        /// <summary>
        /// 支付结果回调
        /// </summary>
        /// <param name="request">支付结果回调请求实体</param>
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        /// <show>false</show>
        Task<bool> PaymentCallbackAsync(PaymentCallbackRequest request);

        /// <summary>
        /// 报名课程
        /// 课程状态：“已审核”才可报名
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <errorCodes>CourseCodeEmpty,CourseNoFound,SignedUp,NotUpdateForCourseStatus</errorCodes>
        Task<bool> SignUpCourseAsync(SignUpCourseRequest request);

        /// <summary>
        /// 取消报名课程
        /// 课程状态：“已审核”取消报名
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        ///  <errorCodes>CourseCodeEmpty,CourseNoFound,NoSignedUp,NotUpdateForCourseStatus</errorCodes>
        Task<bool> CancelSignUpCourseAsync(CancelSignUpCourseRequest request);

        #region 考试

        /// <summary>
        /// 发布考试或者练习试卷接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> PublishCourseExaminationPaperAsync(PublishCourseExaminationPaperRequest request);

        /// <summary>
        /// 查询课程考试列表接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<List<BaseCourseExaminationDTO>> FindCourseExaminationPapersAsync(FindCourseExaminationPapersRequest request);

        /// <summary>
        /// 查询课程考试详情接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<CourseExaminationDTO> FindCourseExaminationPaperByCodeAsync(FindCourseExaminationPaperByCodeRequest request);

        /// <summary>
        /// 提交课程考试接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> SubmitCourseExaminationPaperAsync(SubmitCourseExaminationPaperRequest request);

        /// <summary>
        ///老师查询分页查询学生答题列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<PageResult<BaseStudentExaminationDTO>> FindCourseStudentExaminationPagesAsync(FindCourseStudentExaminationPagesRequest request);
        /// <summary>
        ///老师查询学生课程答题详情 
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<StudentExaminationDetailDTO> FindStudentExaminationByCodeAsync(FindStudentExaminationByCodeRequest request);
        /// <summary>
        /// 提交批阅试卷
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> SubmitReviewStudentExaminationAsync(SubmitReviewStudentExaminationRequest request);

        /// <summary>
        ///分页查看课程自己考试列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<PageResult<BaseStudentExaminationDTO>> FindMyCourseStudentExaminationResultsAsync(FindMyCourseStudentExaminationResultPagesRequest request);


        /// <summary>
        ///学生查看自己考试结果
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<StudentExaminationDetailDTO> FindMyCourseStudentExaminationResultAsync(FindMyCourseStudentExaminationResultRequest request);

        #endregion

        #region 直播课
        /// <summary>
        /// 老师开始直播
        /// </summary>
        /// <param name="request">开始直播请求实体</param>
        /// <docThirdDirectory>教培</docThirdDirectory>  
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        Task<InitiateLiveCourseResult> InitiateLiveCourseAsync(InitiateLiveCourseRequest request);

        /// <summary>
        /// 学员加入房间
        /// </summary>
        /// <param name="request">加入课程请求实体</param>
        /// <docThirdDirectory>教培</docThirdDirectory>  
        /// <errorCodes></errorCodes>
        /// <returns></returns>
        Task<JoinLiveCourseResult> JoinLiveCourseAsync(JoinLiveCourseRequest request);

        /// <summary>
        /// Send HeartRate request in a diagnosis metting
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        Task<LiveHeartRateResult> HeartRateAsync(LiveHeartRateRequest request);

        /// <summary>
        /// 结束课程直播
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        Task<LiveCourseBaseResult> LeaveLiveCourseAsync(JoinLiveCourseRequest request);

        /// <summary>
        /// 设置开启或关闭静音状态
        /// </summary>
        /// <param name="request">设置开启或关闭静音状态的请求</param>
        /// <returns></returns>
        Task<LiveCourseBaseResult> SetMuteLiveCourseAsync(SetMuteLiveCourseRequest request);

        /// <summary>
        /// 开启或关闭视频
        /// </summary>
        /// <param name="request">开启或关闭视频请求</param>
        /// <returns></returns>       
        Task<LiveCourseBaseResult> SetVideoLiveCourseAsync(SetVideoLiveCourseRequest request);

        /// <summary>
        /// 设置设备调参
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> SetControlParameterLiveCourseAsync(ControlParameterInCourseRequest request);
        #endregion


        /// <summary>
        /// 上传远程考核图像
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<bool> UploadRemoteExaminationDataAsync(UploadRemoteExamRequest request);

        /// <summary>
        /// 获取个人远程考核图像列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<IList<RemoteExaminationPageDTO>> GetRemoteExaminationList(TokenRequest request);

        /// <summary>
        /// 学生加入用户组
        /// </summary>
        /// <param name="request"></param>
        /// <errorCodes>UserGroupCodeIsEmpty,UserGroupStudentsIsEmpty,UserGroupNotExists</errorCodes>
        /// <returns></returns>
        Task<bool> AddStudentInCourseUserGroupAsync(AddStudentInCourseUserGroupRequest request);

        /// <summary>
        /// 学习完成处理
        /// </summary>
        /// <param name="request"></param>
        /// <errorCodes>CourseCodeEmpty,UserCodeEmptyError,CourseNoFound</errorCodes>
        /// <returns></returns>
        Task<bool> StudyCompletedAsync(StudyCompletedRequest request);
    }
}