﻿using System.Threading.Tasks;
using System.Collections.Generic;
using WingInterfaceLibrary.DTO.User;
using WingInterfaceLibrary.Request.User;
using WingInterfaceLibrary.Result;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Result.UserFeature;
using WingInterfaceLibrary.DTO.Schedule;
using WingInterfaceLibrary.DTO.Message;

namespace WingInterfaceLibrary.Interface
{
    /// <summary>
    /// 用户个人信息
    /// </summary>
    /// <docFirstDirectory>客户端API</docFirstDirectory>
    /// <docSecondDirectory>个人信息接口</docSecondDirectory>
    public interface IUserService
    {
        /// <summary>
        /// 客户端心跳
        /// </summary>
        /// <param name="request">客户端心跳请求实体</param>
        /// <returns>是否成功</returns>
        /// <value>true</value>
        Task<bool> HeartRateAsync(TokenRequest request);

        /// <summary>
        /// 获取用户信息
        /// </summary>
        /// <param name="request">用户信息请求实体</param>
        /// <returns>用户信息</returns>
        /// <value></value>
        /// <remarks>POST</remarks>
        /// <errorCodes>UserNotExisted</errorCodes>
        Task<UserDTO> GetUserInfoAsync(GetUserInfoRequest request);

        /// <summary>
        /// 修改用户信息
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        /// <value>true</value>
        /// <errorCodes>PhoneBinded,EmailBinded</errorCodes>
        Task<bool> AlterUserInfoAsync(AlterUserInfoRequest request);

        /// <summary>
        /// 根据组织和岗位身份查询用户列表信息
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>用户信息</returns>
        /// <remarks>POST</remarks>
        Task<List<UserExtendDTO>> GetUserListAsync(GetUserListRequest request);

        /// <summary>
        /// 从组织中移除人员
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        /// <value>true</value>
        Task<bool> RemoveUsersFromOrganizationAsync(RemoveUsersFromOrganizationRequest request);

        /// <summary>
        /// 设置用户组织信息
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        /// <value>true</value>
        /// <remarks>POST</remarks>
        Task<bool> SetUserOrganizationInfoAsync(SetUserOrganizationInfoRequest request);

        /// <summary>
        /// 个人中心-修改头像昵称
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>是否成功</returns>
        /// <value>true</value>
        /// <errorCodes>NickNameAndHeadImageUrlEmpty,HeadImageUrlNotValid,NicknameTooLong</errorCodes>
        Task<bool> AlterPersonInfoAsync(AlterPersonInfoRequest request);

        /// <summary>
        /// 获取分享设备用户集合
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>分享设备用户列表</returns>
        /// <remarks>POST</remarks>
        /// <errorCodes>DeviceCodeIsEmpty,NoExistDevice</errorCodes>
        Task<PageResult<ShareDeviceUserDTO>> GetShareDeviceUsersAsync(GetShareDeviceUsersPageRequest request);

        /// <summary>
        /// 根据登录用户获取权限列表
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>权限列表</returns>
        /// <errorCodes>UserNotExisted</errorCodes>
        Task<List<UserFeatureInfoResult>> GetUserFeatureListAsync(TokenRequest request);

        /// <summary>
        /// 根据用户编码获取详情
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns>用户信息</returns>
        /// <errorCodes>UserNotExisted</errorCodes>
        Task<UserInfoByCodeDTO> GetUserByCodeAsync(GetUserByCodeRequest request);


        /// <summary>
        /// 查询是否初始密码
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>结果</returns>
        /// <value>true</value>
        /// <errorCodes>UserNotExisted</errorCodes>
        Task<bool> IsDefaultPasswordAsync(TokenRequest request);

        /// <summary>
        /// 设置通用设置
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>结果</returns>
        /// <value>true</value>
        /// <errorCodes>UserNotExisted</errorCodes>
        Task<bool> SetCommonSettingsAsync(CommonSettingsRequest request);

        /// <summary>
        /// 刷新客户端统计数据
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        /// <show>false</show>
        Task<bool> RefreshStaticticRecordsAsync(RefreshStaticticRecordsRequest request);

        /// <summary>
        /// 查找日程数据请求
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns></returns>
        Task<IList<ClientScheduleDTO>> FindSchedulesAsync(FindSchedulesRequest request);

        /// <summary>
        /// 查询用户消息列表
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>消息列表</returns>
        Task<PageResult<MessageExtendInfoDTO>> GetMessagesPageAsync(QueryMessageListRequest request);

        /// <summary>
        /// 消息送达回调接口
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>false</returns>
        Task<bool> SetMessageDeliveryAsync(SetMessageDeliveryRequest request);

        /// <summary>
        /// 设置消息为已读
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>false</returns>
        Task<bool> BatchSetMessageStatusAsync(SetMessageInfoReqeust request);

        /// <summary>
        /// 查找有效用户总数量
        /// </summary>
        /// <returns></returns>
        Task<long> GetActiveUsersCount();

        /// <summary>
        /// 获取公告内容列表接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<PageResult<AnnouncementExtendInfoDTO>> QueryAnnouncementsByPageAsync(QueryAnnouncementListRequest request);

        /// <summary>
        /// 根据公告Code获取内容接口
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<AnnouncementExtendInfoDTO> GetAnnouncementInfoAsync(GetAnnouncementRequest request);

        /// <summary>
        /// 获取未读消息数量
        /// </summary>
        /// <returns>未读消息编码列表</returns>
        Task<NoReadMessagesDTO> GetNoReadMessagesAsync(GetNoReadMessagesRequest request);


        /// <summary>
        /// 删除单个ws连接
        /// </summary>
        /// <param name="request">登录token信息</param>
        /// <returns>是否成功</returns>
        Task<bool> RemoveUserSingleTokenAsync(RemoveUserSingleTokenRequest request);

    }
}
