using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.DTO.Common;
using WingInterfaceLibrary.Result;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.DTO.Organization;
using WingInterfaceLibrary.DTO;
using WingInterfaceLibrary.LiveConsultation.Consultation;
using WingInterfaceLibrary.DTO.Consultation;
using WingInterfaceLibrary.DTO.User;
using WingInterfaceLibrary.LiveConsultation.QualityControl;
using WingInterfaceLibrary.DTO.Patient;
using WingInterfaceLibrary.OpLog;
using WingInterfaceLibrary.Internal.Request;

namespace WingInterfaceLibrary.LiveConsultation
{
    /// <summary>
    /// 预约会诊接口
    /// </summary>
    /// <docFirstDirectory>客户端API</docFirstDirectory>  
    /// <docSecondDirectory>实时会诊服务接口</docSecondDirectory>   
    /// <docThirdDirectory>预约</docThirdDirectory>  
    public interface ILiveConsultationService
    {
        /// <summary>
        /// 查询上级机构集合
        /// </summary>
        /// <param name="request">查询上级机构集合请求实体</param>
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        Task<IList<OrganizationBaseDTO>> FindParentOrganizationsAsync(FindHigherOrganizationsRequest request);

        /// <summary>
        /// 查询基层机构集合
        /// </summary>
        /// <param name="request">查询基层机构集合请求实体</param>
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        Task<IList<OrganizationBaseDTO>> FindGrassRootsOrganizationsAsync(FindGrassRootsOrganizationsRequest request);

        /// <summary>
        /// 查询机构专家集合
        /// </summary>
        /// <param name="request">查询机构专家集合请求实体</param>
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        Task<IList<OrganizationExpertDTO>> FindOrganizationExpertsAsync(FindOrganizationExpertsRequest request);

        /// <summary>
        /// 申请会诊
        /// </summary>
        /// <param name="request">申请会诊请求实体</param>
        /// <returns>会诊编码</returns>
        /// <value>ConsultationCodeXXXX</value>
        /// <errorCodes>UserNotFind,PatientDataIsEmpty,PatientNameIsEmpty,PatientPhoneIsEmpty,DeviceCodeIsEmpty,ConsultationTimeGreaterThanCurrentTime,ExpertOrganizationCodeIsEmpty,ConsultationExpertNotExist,ApplyUserCodeIsEmpty,ConsultationApplyUserIllegal,PatientCodeIsEmpty,ConsultationTimeIsEmpty,PhoneAndNameExistError</errorCodes>
        Task<string> ApplyConsultationAsync(ApplyConsultationRequest request);

        /// <summary>
        /// 修改会诊信息
        /// </summary>
        /// <param name="request">申请会诊请求实体</param>
        /// <returns>是否成功</returns>
        /// <value>true</value>
        /// <errorCodes>UserNotFind,ConsultationNorightError,ConsultationStatusError,DeviceCodeIsEmpty,ConsultationTimeGreaterThanCurrentTime,ExpertOrganizationCodeIsEmpty,ConsultationExpertNotExist,ApplyUserCodeIsEmpty,ConsultationExpertNotExist,ConsultationApplyUserIllegal</errorCodes>
        Task<bool> UpdateConsultationAsync(UpdateConsultationRequest request);

        /// <summary>
        /// 完善会诊信息
        /// </summary>
        /// <param name="request">完善会诊信息请求实体</param>
        /// <returns>是否成功</returns>
        /// <value>true</value>
        /// <errorCodes>UserNotFind,ConsultationNorightError,ConsultationStatusError,PatientCodeIsEmpty,UpdatePatientInfoFail</errorCodes>
        Task<bool> ImproveConsultationInfoAsync(ImproveConsultationInfoRequest request);

        /// <summary>查询助理服务专家集合</summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<IList<OrganizationExpertDTO>> FindAssistantExpertsAsync(FindAssistantExpertsRequest request);

        /// <summary>
        /// 分页查询会诊记录
        /// </summary>
        /// <param name="request">分页查询会诊记录请求实体</param>
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        Task<PageResult<ConsultationPageDTO>> FindConsultationsByPageAsync(FindConsultationByPageRequest request);

        /// <summary>
        /// 会诊大列表导出
        /// </summary>
        /// <param name="request">会诊大列表导出请求实体</param>
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        Task<ExportConsultationDataResult> ExportConsultationsAsync(ExportConsultationsRequest request);

        /// <summary>
        /// 查询会诊详情
        /// </summary>
        /// <param name="request">查询会诊详情请求实体</param>
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        Task<ConsultationDetailDTO> FindConsultationDetailAsync(FindConsultationDetailRequest request);

        /// <summary>
        /// 批准会诊申请
        /// </summary>
        /// <param name="request">批准会诊申请请求实体</param>
        /// <returns></returns>
        /// <value>true</value>
        /// <errorCodes>UserNotFind,ConsultationTimeGreaterThanCurrentTime,ConsultationExpertNotExist,ConsultationNorightError,ConsultationStatusError</errorCodes>
        Task<bool> ApprovalConsultationAsync(ApprovalConsultationRequest request);

        /// <summary>
        /// 拒绝申请的会诊请求
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <value>true</value>
        /// <errorCodes>ConsultationStatusError,ConsultationNorightError</errorCodes>
        Task<bool> RejectApplyConsultationAsync(RejectApplyConsultationRequest request);

        /// <summary>
        /// 撤回会诊申请
        /// </summary>
        /// <param name="request">撤回会诊申请请求实体</param>
        /// <returns></returns>
        /// <value>true</value>
        /// <errorCodes>ConsultationStatusError,ConsultationNorightError</errorCodes>
        Task<bool> RevokeConsultationAsync(RevokeConsultationRequest request);

        /// <summary>
        /// 删除会诊记录
        /// </summary>
        /// <param name="request">删除会诊记录请求实体</param>
        /// <returns></returns>
        /// <value>true</value>
        /// <errorCodes>ConsultationStatusError,ConsultationNorightError</errorCodes>
        Task<bool> DeleteConsultationAsync(DeleteConsultationRequest request);

        /// <summary>
        /// 保存预约会诊图像
        /// 累加保存
        /// </summary>
        /// <param name="request">保存预约会诊图像请求实体</param>
        /// <returns></returns>
        /// <value>true</value>
        Task<bool> UpdateConsultationFilesInfoAsync(UpdateConsultationFilesInfoRequest request);

        /// <summary>
        /// 查询自己所在的机构专家助理集合
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<IList<ConsultationAssistantDTO>> FindOrganizationAssistantsAsync(FindOrganizationAssistantsRequest request);

        /// <summary>
        /// 查询自己所在的机构认证医师集合
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<IList<ConsultationAssistantDoctorDTO>> FindOrganizationAssistantDoctorsAsync(FindOrganizationAssistantDoctorsRequest request);

        /// <summary>
        /// 更新设置自己的专家助理
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <value>true</value>
        /// <errorCodes>NoExpertFeature,AssistantNotExisted,NotExpertAssistant,NotCertifiedPhysician</errorCodes>
        Task<bool> UpdateMyOrganizationAssistantAsync(UpdateMyOrganizationAssistantRequest request);

        /// <summary>
        ///查询已创建已登记的病人
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        Task<PageResult<ClientPatientInfoBaseDTO>> FindConsultationPatientAsync(FindConsultationPatientPageRequest request);

        /// <summary>
        /// 根据会诊单号查询会诊中可直接切换的会诊单列表
        /// </summary>
        /// <param name="request">根据会诊单号查询会诊中可直接切换的会诊单列表</param>
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        Task<IList<ConsultationPageDTO>> FindCanSwitchConsultationsAsync(FindCanSwitchConsultationsRequest request);

        /// <summary>
        /// 开始会诊
        /// </summary>
        /// <param name="request">开始会诊请求实体</param>
        /// <docThirdDirectory>会诊</docThirdDirectory>  
        /// <returns></returns>
        /// <errorCodes>SaveInitiatorFailed,ConsultationNotExisted,ConsultationExpertOrgError,ReservationNotReviewed,ApplyerNotOnline,ApplyerIsBusy,ConsultationNotOneOnLine,ConsultationDBError,ConsultationExpertIsEmpty,ConsultationInProcess,ReferralOrganizationsIsEmpty,EmergencyDoctorsIsEmpty</errorCodes>
        Task<InitiateLiveConsultationResult> InitiateLiveConsultationAsync(InitiateLiveConsultationRequest request);

        /// <summary>
        /// 会诊过程中邀请其他成员
        /// </summary>
        /// <param name="request">会诊过程中邀请其他成员请求实体</param>
        /// <docThirdDirectory>会诊</docThirdDirectory> 
        /// <errorCodes>LiveConsultationNotFound,ConsultationNotInProcess,InviterNotOnLine,InviterIsBusy</errorCodes>
        /// <returns></returns>
        Task<InviteInLiveConsultationResult> InviteInLiveConsultationAsync(InviteInLiveConsultationRequest request);

        /// <summary>
        /// 取消会诊过程邀请中的成员
        /// </summary>
        /// <param name="request">取消会诊过程邀请中的成员请求实体</param>
        /// <docThirdDirectory>会诊</docThirdDirectory>  
        /// <errorCodes>LiveConsultationNotFound,ConsultationNotInProcess</errorCodes>
        /// <returns></returns>
        Task<CancelInvitingInLiveConsultationResult> CancelInvitingInLiveConsultationAsync(CancelInvitingInLiveConsultationRequest request);

        /// <summary>
        /// 拒绝会诊中邀请
        /// </summary>
        /// <param name="request">The request</param>
        /// <docThirdDirectory>会诊</docThirdDirectory>  
        /// <errorCodes>LiveConsultationNotFound</errorCodes>
        /// <returns></returns>
        Task<RejectLiveConsultationResult> RejectInviteLiveConsultationAsync(RejectLiveConsultationRequest request);

        /// <summary>
        /// 会诊中加入房间
        /// </summary>
        /// <param name="request">The request</param>
        /// <docThirdDirectory>会诊</docThirdDirectory>  
        /// <errorCodes>LiveConsultationNotFound,ConsultationNotInProcess,JoinedByOtherDevice</errorCodes>
        /// <returns></returns>
        Task<JoinLiveConsultationResult> JoinInLiveConsultationAsync(JoinLiveConsultationRequest request);


        /// <summary>
        /// 取消会诊
        /// </summary>
        /// <param name="request">取消会诊请求实体</param>
        /// <docThirdDirectory>会诊</docThirdDirectory>  
        /// <errorCodes>LiveConsultationNotFound,LiveConsultationCancelled,NotConsultationInitiator</errorCodes>
        /// <returns></returns>
        Task<CancelLiveConsultationResult> CancelLiveConsultationAsync(CancelLiveConsultationRequest request);

        /// <summary>
        /// 接受会诊
        /// </summary>
        /// <param name="request">接受会诊请求实体</param> 
        /// <docThirdDirectory>会诊</docThirdDirectory>  
        /// <errorCodes>LiveConsultationNotFound,LiveConsultationCancelled,LiveConsultationTimeout,LivMemberNotFound,JoinedByOtherDevice</errorCodes>
        /// <returns></returns>
        Task<AcceptLiveConsultationResult> AcceptLiveConsultationAsync(AcceptLiveConsultationRequest request);

        /// <summary>
        /// 拒绝会诊
        /// </summary>
        /// <param name="request">拒绝会诊请求实体</param>
        /// <docThirdDirectory>会诊</docThirdDirectory>  
        /// <errorCodes>LiveConsultationNotFound,LivMemberNotFound</errorCodes>
        /// <returns></returns>
        Task<RejectLiveConsultationResult> RejectLiveConsultationAsync(RejectLiveConsultationRequest request);

        /// <summary>
        /// 加入房间
        /// </summary>
        /// <param name="request">加入房间请求实体</param>
        /// <docThirdDirectory>会诊</docThirdDirectory>  
        /// <show>false</show>
        /// <returns></returns>
        Task<JoinLiveConsultationResult> JoinLiveConsultationAsync(JoinLiveConsultationRequest request);

        /// <summary>
        /// 离开房间
        /// </summary>
        /// <param name="request">离开房间请求实体</param>
        /// <docThirdDirectory>会诊</docThirdDirectory>  
        /// <errorCodes>LiveConsultationNotFound</errorCodes>
        /// <returns></returns>
        Task<LeaveLiveConsultationResult> LeaveLiveConsultationAsync(LeaveLiveConsultationRequest request);

        /// <summary>
        /// 设置开启或关闭静音状态
        /// </summary>
        /// <param name="request">设置开启或关闭静音状态的请求</param>
        /// <docThirdDirectory>会诊</docThirdDirectory>  
        /// <errorCodes>LiveConsultationNotFound,ConsultationNotInProcess</errorCodes>
        /// <returns></returns>
        Task<MuteLiveConsultationResult> MuteLiveConsultationAsync(MuteLiveConsultationRequest request);

        /// <summary>
        /// 设置开启或关闭视频
        /// </summary>
        /// <param name="request">设置开启或关闭视频请求</param>
        /// <docThirdDirectory>会诊</docThirdDirectory>  
        /// <errorCodes>LiveConsultationNotFound,ConsultationNotInProcess</errorCodes>
        /// <returns></returns>
        Task<SwitchLiveConsultationVideoResult> SwitchLiveConsultationVideoAsync(SwitchLiveConsultationVideoRequest request);

        /// <summary>
        /// 会诊心跳
        /// </summary>
        /// <param name="request">会诊心跳请求实体</param>
        /// <docThirdDirectory>会诊</docThirdDirectory> 
        /// <errorCodes>LiveConsultationNotFound,ReservationNotReviewed,ConsultationEnded</errorCodes>
        /// <returns></returns>
        Task<LiveConsultationHeartRateResult> HeartRateAsync(LiveConsultationHeartRateRequest request);

        /// <summary>
        /// 发送互动白板操作数据
        /// </summary>
        /// <param name="request">发送互动白板操作数据请求实体</param>
        /// <docThirdDirectory>会诊</docThirdDirectory>  
        /// <errorCodes>LiveConsultationNotFound</errorCodes>
        /// <returns></returns>
        Task<bool> SendInteractiveBoardDataAsync(SendInteractiveBoardDataRequest request);

        /// <summary>
        /// 添加随访信息
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <errorCodes>AuthenticationError</errorCodes>
        /// <show>true</show>
        Task<bool> AddFollowUpVisitInfoAsync(AddFollowUpVisitInfoRequest request);

        /// <summary>
        /// 删除随访信息
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <errorCodes>AuthenticationError,ConsultationStatusError</errorCodes>
        /// <show>false</show>
        Task<bool> DeleteFollowUpVisitInfoAsync(DeleteFollowUpVisitInfoRequest request);

        /// <summary>
        /// 修改随访信息
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <errorCodes>AuthenticationError</errorCodes>
        /// <show>false</show>
        Task<bool> UpdateFollowUpVisitInfoAsync(UpdateFollowUpVisitInfoRequest request);

        /// <summary>
        /// 查询随访信息
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <errorCodes>AuthenticationError</errorCodes>
        /// <show>true</show>
        Task<List<FollowUpVisitDTO>> GetFollowUpVisitInfoAsync(GetFollowUpVisitInfoRequest request);

        /// <summary>
        /// 查询随访信息详情
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <errorCodes>AuthenticationError,FollowUpVisitInfoNotFound</errorCodes>
        /// <show>false</show>
        Task<FollowUpVisitDTO> GetFollowUpVisitInfoDetailAsync(GetFollowUpVisitInfoDetailRequest request);

        /// <summary>
        /// 设置需要随访
        /// </summary>
        /// <returns></returns>
        /// <errorCodes>AuthenticationError</errorCodes>
        /// <show>false</show>
        Task<bool> SetNeedFollowUpVisitAsync(SetNeedFollowUpVisitRequest request);

        /// <summary>
        /// 新增会诊评价
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <errorCodes>AuthenticationError,ConsultationStatusError</errorCodes>
        /// <show>true</show>
        Task<bool> AddConsultationEvaluateAsync(AddConsultationEvaluateRequest request);

        /// <summary>
        /// 删除会诊评价
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <errorCodes>AuthenticationError</errorCodes>
        /// <show>false</show>
        Task<bool> DeleteConsultationEvaluateAsync(DeleteConsultationEvaluateRequest request);

        /// <summary>
        /// 修改会诊评价
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <errorCodes>AuthenticationError</errorCodes>
        /// <show>false</show>
        Task<bool> UpdateConsultationEvaluateAsync(UpdateConsultationEvaluateRequest request);

        /// <summary>
        /// 查询会诊评价详情
        /// </summary>
        /// <param name="request">The request</param>
        /// <returns></returns>
        /// <errorCodes>AuthenticationError,ConsultationEvaluateNotFound</errorCodes>
        /// <show>true</show>
        Task<ConsultationEvaluateDTO> GetConsultationEvaluateDetailAsync(GetConsultationEvaluateRequest request);

        /// <summary>
        /// 查询扫查部位列表 
        /// </summary>
        /// <param name="request">token请求实体</param>
        /// <returns></returns>
        Task<List<string>> FindScanPositionsAsync(TokenRequest request);

        /// <summary>
        /// 接受或拒绝会诊邀请
        /// </summary>
        /// <param name="request">The request</param>
        /// <errorCodes>ConsultationNotExisted</errorCodes>
        /// <returns></returns>
        Task<bool> AcceptInvitationAsync(AcceptInvitationRequest request);

        /// <summary>
        /// 急诊申请
        /// </summary>
        ///  <param name="request">The request</param>
        /// <errorCodes>UniqueCodeIsEmpty,UniqueCodeIsError,EmergencyCodeError</errorCodes>
        Task<string> ApplyEmergencyTreatment(ApplyEmergencyTreatmentRequest request);

        /// <summary>
        /// 调节设备参数
        /// </summary>
        /// <param name="request">发送调节设备参数请求实体</param>
        /// <docThirdDirectory>会诊</docThirdDirectory>  
        /// <errorCodes>ControllingParameterByOtherUser,LiveConsultationNotFound,LiveMemberNotFound,NoExistDevice</errorCodes>
        /// <returns></returns>
        Task<bool> ControlParameterAsync(ControlDeviceParameterInConsultationRequest request);


        /// <summary>
        /// 获取用户繁忙状态
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        /// <show>false</show>
        Task<UserStatusEnum> GetUserStatusAsync(GetUserStatusRequest request);

        /// <summary>
        /// 查询会诊图像接口
        /// </summary>
        /// <param name="request">查询图像请求实体</param>
        /// <returns></returns>
        /// <errorCodes></errorCodes>
        Task<IList<ConsultationImagesDTO>> GetConsultationFilesAsync(FindConsultationDetailRequest request);

        /// <summary>
        /// 会诊中获取可邀请用户信息列表
        /// </summary>
        /// <param name="request">请求体</param>
        /// <errorCodes>LiveConsultationNotFound,ConsultationNotInProcess</errorCodes>
        /// <returns>用户信息列表</returns>
        Task<List<UserExtendDTO>> GetInviteableUserListAsync(GetInviteableUserListRequest request);

        /// <summary>
        /// 切换会诊房间
        /// </summary>
        /// <param name="request">切换会诊房间请求实体</param>
        /// <docThirdDirectory>会诊</docThirdDirectory>  
        /// <errorCodes>LiveConsultationNotFound,ConsultationNotInProcess,CurrentConsultationNotFound,CurrentConsultationInProcess,ConsultationDBError,ChangeConsultationNoFeature,OperatorNoJoined</errorCodes>
        /// <returns></returns>
        Task<ChangeConsultationResult> ChangeConsultationAsync(ChangeConsultationRequest request);

        /// <summary>
        /// 更改调参状态
        /// </summary>
        /// <param name="request">请求实体</param>
        /// <returns>true</returns>
        /// <show>false</show>
        Task<bool> ChangeConsultationControllingStateAsync(ChangeConsultationControllingStateRequest request);

        /// <summary>
        /// 设备会诊中加入房间
        /// </summary>
        /// <param name="request">The request</param>
        /// <docThirdDirectory>会诊</docThirdDirectory>  
        /// <errorCodes>LiveConsultationNotFound,ConsultationNotInProcess,JoinedByOtherDevice</errorCodes>
        /// <returns></returns>
        Task<LiveConsultationMember> DeviceJoinInLiveConsultationAsync(JoinLiveConsultationRequest request);
    }
}